#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________

__all__ = ['Client']

import Pyro4.core
import Queue
import os, socket
from pyutilib.pyro.util import *
from Pyro4.errors import NamingError

class Client(object):

    def __init__(self, group=":PyUtilibServer", type=None, host=None):
        self.type=type
        self.id = 0
        Pyro4.core.initClient()
        self.ns = get_nameserver(host)
        if self.ns is None:
            raise RuntimeError, "Client failed to locate Pyro name server on the network!"
        print 'Finding Pyro dispatcher object...'
        try:
            self.URI=self.ns.resolve(group+".dispatcher")
            print 'Dispatcher Object URI:',self.URI
        except NamingError,x:
            print 'Couldn\'t find object, nameserver says:',x
            raise SystemExit
        self.set_group(group)
        self.CLIENTNAME = "%d@%s" % (os.getpid(), socket.gethostname())
        print "This is client",self.CLIENTNAME

    def set_group(self, group):
        self.dispatcher = Pyro4.core.getProxyForURI(self.URI)

    def add_task(self, task):
        if task.id is None:
            task.id = self.CLIENTNAME + "_" + str(self.id)
            self.id += 1
        else:
            task.id = self.CLIENTNAME + "_" + str(task.id)
        if self.type is not None:
           print "Adding task",task.id,"to dispatcher queue with type=",self.type
        else:
           print "Adding task",task.id,"to dispatcher queue"
        self.dispatcher.add_task(task, type=self.type)

    def get_result(self):
        try:
            return self.dispatcher.get_result(type=self.type)
        except Queue.Empty:
            return None

    def num_tasks(self):
        return self.dispatcher.num_tasks(type=self.type)

    def num_results(self):
        return self.dispatcher.num_results(type=self.type)

